#include "esd.h"
#include "config.h"
#include "soundcard.h"

#include <errno.h>
#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "support_win32.h"

/*******************************************************************/
/* display available devices */
#define ARCH_esd_audio_devices
const char * esd_audio_devices()
{
    return "(default audio device)";
}

/*******************************************************************/
/* open the audio device */
#define ARCH_esd_audio_open
int esd_audio_open()
{
	int value = 0, test = 0;
	const int audio_fd = 10;
    int mode = O_WRONLY;
	int err = 0;
	int num_in = 0;
	int num_out = 0;

    /* if recording, set for full duplex mode */
    if ( (esd_audio_format & ESD_MASK_FUNC) == ESD_RECORD )
        mode = O_RDWR;

	// Work out initial sample format & frequency, /dev/dsp defaults
	audioformat_ = AFMT_U8;
	audiofreq_ = 8000;
	audiobits_ = 8;
	audiochannels_ = 1;

	switch (mode)
	{
	case O_RDWR:
		if ((num_in = waveInGetNumDevs ()) == 0)
			err = -1;
		/* Fall through */
	case O_WRONLY:
		if ((num_out = waveOutGetNumDevs ()) == 0)
			err = -1;
		break;
	}

	if (err != 0)
		return -1;
	

    /* TODO: check that this is allowable */
    /* set for full duplex operation, if recording */
    if ( (esd_audio_format & ESD_MASK_FUNC) == ESD_RECORD ) {
        ioctl( SNDCTL_DSP_SETDUPLEX, 0 );
    }

    /* set the sound driver fragment size and number */
#if !defined(__powerpc__) /* does not exist on powerpc */
    /* fragment = max_buffers << 16 + log2(buffer_size), (256 16) */
    value = test = ( 0x0100 << 16 ) + 0x0008;
    if (ioctl(SNDCTL_DSP_SETFRAGMENT, &test) == -1)
    {   /* Fatal error */
        perror( "SNDCTL_DSP_SETFRAGMENT" );
        // TODO: close( afd );
        esd_audio_fd = -1;
        return( -1 );
    }
    if ( 0 /* value != test */ ) /* TODO: determine the real test */
    {   /* The device doesn't support the requested audio format. */
        fprintf( stderr, "unsupported fragment size: %d\n", value );
        // TODO: close( afd );
        esd_audio_fd = -1;
        return( -1 );
    }
#endif /* #if !defined(__powerpc__) */

    /* set the sound driver audio format for playback */
#if defined(__powerpc__)
    value = test = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 )
        ? /* 16 bit */ AFMT_S16_NE : /* 8 bit */ AFMT_U8;
#else /* #if !defined(__powerpc__) */
    value = test = ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 )
        ? /* 16 bit */ AFMT_S16_LE : /* 8 bit */ AFMT_U8;
#endif /* #if !defined(__powerpc__) */

    if (ioctl(SNDCTL_DSP_SETFMT, &test) == -1)
    {   /* Fatal error */
        perror("SNDCTL_DSP_SETFMT");
        // TODO: close( afd );
        esd_audio_fd = -1;
        return( -1 );
    }

    ioctl(SNDCTL_DSP_GETFMTS, &test);
    if ( !(value & test) ) /* TODO: should this be if ( value XOR test ) ??? */
    {   /* The device doesn't support the requested audio format. */
        fprintf( stderr, "unsupported sound format: %d\n", esd_audio_format );
        // TODO: close( afd );
        esd_audio_fd = -1;
        return( -1 );
    }

    /* set the sound driver number of channels for playback */
    value = test = ( ( ( esd_audio_format & ESD_MASK_CHAN) == ESD_STEREO )
        ? /* stereo */ 1 : /* mono */ 0 );
    if (ioctl(SNDCTL_DSP_STEREO, &test) == -1)
    {   /* Fatal error */
        perror( "SNDCTL_DSP_STEREO" );
        // TODO: close( afd );
        esd_audio_fd = -1;
        return( -1 );
    }

    /* set the sound driver number playback rate */
    test = esd_audio_rate;
    if ( ioctl(SNDCTL_DSP_SPEED, &test) == -1)
    { /* Fatal error */
        perror("SNDCTL_DSP_SPEED");
        // TODO: close( afd );
        esd_audio_fd = -1;
        return( -1 );
    }

    /* see if actual speed is within 5% of requested speed */
    if( fabs( (double) test - esd_audio_rate ) > esd_audio_rate * 0.05 )
    {
        fprintf( stderr, "unsupported playback rate: %d\n", esd_audio_rate );
        // TODO: close( afd );
        esd_audio_fd = -1;
        return( -1 );
    }

	open_flags_ = mode;
	esd_audio_fd = audio_fd;
    return audio_fd;
}

/*******************************************************************/
/* make the sound device quiet for a while */
#define ARCH_esd_audio_pause
void esd_audio_pause()
{
	// TODO: esd_audio_pause
    return;
}

/*******************************************************************/
/* close the audio device */
#define ARCH_esd_audio_close
void esd_audio_close()
{
    //close( esd_audio_fd );
	// TODO: esd_audio_close
    return;
}




/*******************************************************************/
/* dump a buffer to the sound device */
#define ARCH_esd_audio_write
int esd_audio_write( void *ptr, int len )
{
  int len_s = len;
  const char *ptr_s = (const char *) (ptr);
  audio_out_ptr audio_out = NULL;

  debug_printf ("ptr=%08x len=%d", ptr, len);

  if (ptr == NULL)
	  return 0;

  if (!(audio_out = audio_out_get_ptr()))
    if (IS_WRITE ())
      {
	debug_printf ("Allocating");
	if (!(audio_out = audio_out_new()))
	  return -1;

	/* check for wave file & get parameters & skip header if possible. */

	if (audio_out_parsewav (audio_out, &ptr_s, &len_s,
				  audiofreq_, audiobits_, audiochannels_))
	  debug_printf ("=> ptr_s=%08x len_s=%d", ptr_s, len_s);
      }
    else
      {
	set_errno (EBADF); // device was opened for read?
	return -1;
      }

  /* Open audio device properly with callbacks.
     Private parameters were set in call to parsewav.
     This is a no-op when there are successive writes in the same process */
  if (!audio_out_start (audio_out))
    {
      set_errno (EIO);
      return -1;
    }

  audio_out_write (audio_out, ptr_s, len_s);
  return len;
}

/*******************************************************************/
/* read a chunk from the sound device */
#define ARCH_esd_audio_read
int esd_audio_read( void *buffer, int buf_size )
{
    //return read( esd_audio_fd, buffer, buf_size );
	// TODO: esd_audio_read
	return -1;
}

/*******************************************************************/
/* flush the audio buffer */
#define ARCH_esd_audio_flush
void esd_audio_flush()
{
    //fsync( esd_audio_fd );
	// TODO: esd_audio_flush
    return;
}
