#include <winsock2.h>
#include <io.h>
#include <assert.h>
#include "sfds.h"

#define SFD_MAP_SIZE	256
#define SFD_SOCKET_INVALID	-1
/*
 * This socket map just contains slots to put SOCKET handles in.
 * The sfd number is calculated by SFD_SOCKET_BASE + i where i is sfd_socket_map[i],
 * so the SOCKET in slot zero gets sfd number (SFD_SOCKET_BASE + 0)
 */
static SOCKET sfd_socket_map[SFD_MAP_SIZE] = { 0 };
static int sfd_socket_map_init = 0;

/* For a real fd or SOCKET, allocate an sfd (will be the same for real fds) */
int
allocate_sfd(int fd_or_socket)
{
	int slot = SFD_SOCKET_INVALID;
	int i;

	/* Init the map once */
	if (!sfd_socket_map_init)
	{
		sfd_socket_map_init = 1;
		for (i = 0; i < SFD_MAP_SIZE; ++i)
		{
			sfd_socket_map[i] = SFD_SOCKET_INVALID;
		}
	}

	/* Nothing to do if this is an fd */
	if (SFD_IS_REAL_FD(fd_or_socket))
	{
		return fd_or_socket;
	}

	/* Find an open slot */
	for (i = 0; i < SFD_MAP_SIZE; ++i)
	{
		/* Is this slot open? */
		if (sfd_socket_map[i] == SFD_SOCKET_INVALID)
		{
			slot = i;
			break;
		}
	}
	
	/* Bail if no slot found */
	assert(slot != SFD_SOCKET_INVALID);

	/* Save the handle */
	sfd_socket_map[slot] = (SOCKET) fd_or_socket;

	/* Return the slot plus the base as the sfd */
	return (SFD_SOCKET_BASE + slot);
}

/* For an sfd, get the real handle behind it */
int
sfd_to_real(int sfd, sfd_type *type)
{
	int dummy_type;
	SOCKET sock;

	/* Check if caller doesn't care about type */
	if (type == NULL)
	{
		type = &dummy_type;
	}

	/* Real fds map back to themselves */
	if (SFD_IS_REAL_FD(sfd))
	{
		*type = SFD_TYPE_FD;
		return sfd;
	}

	/* If the map has a value at this slot it is the SOCKET for this sfd */
	sock = sfd_socket_map[sfd - SFD_SOCKET_BASE];
	assert(sock != SFD_TYPE_NONE);

	return (int) sock;
}

/* Free an sfd from the map */
void
free_sfd(int sfd)
{
	SOCKET sock;

	/* Nothing to do if the sfd is a real fd */
	if (SFD_IS_REAL_FD(sfd))
	{
		return;
	}

	/* Get the value at this slot */
	sock = sfd_socket_map[sfd - SFD_SOCKET_BASE];

	/* Bail if this is a double-free */
	assert(sock != SFD_SOCKET_INVALID);

	/* Blank the slot */
	sfd_socket_map[sfd - SFD_SOCKET_BASE] = SFD_SOCKET_INVALID;
}
