#ifndef _LIBWINSOCKHELP_SOCKET_H_
#define _LIBWINSOCKHELP_SOCKET_H_ 1

/* Include the original header */
#include <winsock2.h>


/* Declare our indirection functions */
int WSHELPsocket (int af, int type, int protocol);
int WSHELPsetsockopt (int sfd, int level, int optname, const char* optval, int optlen);
int WSHELPioctlsocket(int sfd, long cmd, u_long* argp);
int WSHELPlisten (int sfd, int backlog);
int WSHELPbind (int sfd, const struct sockaddr *name, int namelen);
int WSHELPconnect (int sfd, const struct sockaddr* name, int namelen);
int WSHELPaccept(int sfd, struct sockaddr* addr, int* addrlen);
int WSHELPselect (int sfds, fd_set* readfds, fd_set* writefds, fd_set* exceptfds, const struct timeval *timeout);

/* Other helpers */
void WSHELPinitialize();

/* Redirect callers of socket functions to use our indirection functions */
#define socket(af, type, protocol)									WSHELPsocket(af, type, protocol)
#define setsockopt(sfd, level, optname, optval, optlen)				WSHELPsetsockopt(sfd, level, optname, optval, optlen)
#define ioctlsocket(sfd, cmd, argp)									WSHELPioctlsocket(sfd, cmd, argp)
#define listen(sfd, backlog)										WSHELPlisten(sfd, backlog)
#define bind(sfd, name, namelen)									WSHELPbind(sfd, name, namelen)
#define connect(sfd, name, namelen)									WSHELPconnect(sfd, name, namelen)
#define accept(sfd, addr, addrlen)									WSHELPaccept(sfd, addr, addrlen)
#define select(sfds, readsfds, writesfds, exceptsfds, timeout)		WSHELPselect(sfds, readsfds, writesfds, exceptsfds, timeout)


/* Include the original header */
#include <io.h>

/* Declare our indirection functions */
int WSHELPread(int sfd, void *dst, unsigned int max);
int WSHELPwrite(int sfd, const void *buf, unsigned int max);
int WSHELPclose(int sfd);

/* Redirect callers of io functions to use our indirection functions */
#define read(fd, dst, max)		WSHELPread(fd, dst, max)
#define write(fd, buf, max)		WSHELPwrite(fd, buf, max)
#define close(fd)				WSHELPclose(fd)

/* Errno helpers */
#define ENETDOWN		WSAENETDOWN
#define EAFNOSUPPORT	WSAEAFNOSUPPORT
#define	EINPROGRESS		WSAEINPROGRESS
#define EXX				WSAEMFILE
#define EXX1			WSAENOBUFS
#define EPROTONOSUPPORT	WSAEPROTONOSUPPORT
#define EPROTOTYPE		WSAEPROTOTYPE
#define	ESOCKTNOSUPPORT	WSAESOCKTNOSUPPORT
#define EADDRINUSE		WSAEADDRINUSE
// in errno.h #define EINVAL			WSAEINVAL
#define EISCONN			WSAEISCONN
#define ENOTSOCK		WSAENOTSOCK
#define EOPNOTSUPP		WSAENOTSUPP
// in errno.h #define EINTR			WSAEINTR
#define EALREADY		WSAEALREADY
#define	ECONNREFUSED	WSAECONNREFUSED
// in errno.h #define	EFAULT			WSAEFAULT
#define ENOTUNREACH		WSAENOTUNREACH
#define EHOSTUNREACH	WSAEHOSTUNREACH
#define ETIMEDOUT		WSAETIMEDOUT
#define EWOULDBLOCK		WSAEWOULDBLOCK
#define	EACCES			WSAEACCESS
#define ECONNRESET		WSAECONNRESET
#define ENOPROTOOPT		WSAENOPROTOOPT
#endif
